
#################################
#################################
## Gene set analysis module
#################################
#################################
run.gene.set.analyses = function(DEresults,gene.sets,gene.set.names,deregscores=NULL,saveresults=TRUE,pval.adjustment,
                                 log,plottypearg,path.results,path.inc,path.to.csvs,path.to.GSA.results)
{
  print ("Starting gene set analysis (GSA)")
  cat("LOG:Starting gene set analysis (GSA)",file=log,sep='\n\n',append=TRUE)
  cat("document.write('<p>Starting gene set analysis (GSA)</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)

  # remove columns of gene set matrix that are empty:
  empty.gene.sets = colSums(gene.sets)==0
  #gene.sets = gene.sets[,!empty.gene.sets]
  #gene.set.names = gene.set.names[!empty.gene.sets]
 
  nvars = DEresults$npredictors
  pvals=as.data.frame(DEresults$pvals[,1:nvars]); dimnames(pvals)[[2]]=dimnames(DEresults$pvals)[[2]][1:nvars]
  ests=as.data.frame(DEresults$ests[,1:nvars]); dimnames(ests)[[2]]=dimnames(DEresults$ests)[[2]][1:nvars]
  ngenes = dim(DEresults$pvals)[1]
  newtermnames = DEresults$newtermnames
  newtermnames.linebreak = DEresults$newtermnames.linebreak
  # revise gene.sets so the genes match those of DEresults:
  gene.sets = gene.sets[match(dimnames(pvals)[[1]],dimnames(gene.sets)[[1]]),]
  K = dim(gene.sets)[2]
  covariates = dimnames(DEresults$ests)[[2]][1:nvars]
  
  # matrices of statistics to save:
  global.sig.stats = matrix(NA,dim(gene.sets)[2],length(covariates))
  dimnames(global.sig.stats)[[1]]=gene.set.names
  dimnames(global.sig.stats)[[2]]=newtermnames[covariates]
  global.sig.stats.updown = global.sig.stats

  
  for(k in 1:K)
  {
    print(paste("Creating gene set analysis (GSA) plots for gene set ",k,"/", K, sep=""))
    cat(paste("LOG:Creating gene set analysis (GSA) plots for gene set ",k,"/", K, sep=""),file=log,sep='\n\n',append=TRUE)
    cat(paste("document.write('<p>Creating gene set analysis (GSA) plots for gene set ",k,"/", K, "</p>');", sep=""), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
    
    # for each covariate saved in DEresults
    for(i in 1:nvars)
    {  
      pvals.all = pvals[,i]
      tstats.all = ests[,i]/DEresults$ses[,i]
      
      
      pvals.path = pvals.all[gene.sets[,k]==1]
      
      # calculate global significance stat:
      global.sig.stats[k,i] = round(sqrt(mean((tstats.all^2)[gene.sets[,k]==1])),4)
      global.sig.stats.updown[k,i] = mean((sign(tstats.all)*(tstats.all^2))[gene.sets[,k]==1])
      global.sig.stats.updown[k,i] = round(sign(global.sig.stats.updown[k,i])*sqrt(abs(global.sig.stats.updown[k,i])),4)
      
      if(saveresults)
      {
        # draw a volcano plot
        for(r in 1:length(plottypearg)){
          plottype=plottypearg[r];
          tempfilename = drawplot(paste(path.to.GSA.results,"//volcano plot - ",make.names(covariates[i])," - ",gene.set.names[k],sep=""),plottype,width=1.3,height=1.3)
          tempfilename=gsub(path.results,"results",tempfilename)
          
          volcanocol = rep("grey",ngenes)
          volcanocol[gene.sets[,k]==1]=codecols2[3]
          volcanopch = rep(1,ngenes)
          volcanopch[gene.sets[,k]==1]=16
          
          if(is.element(DEresults$pval.adjustment,c("BH","BY")))
          {
            pvals.a = DEresults$pvals.adjusted[,i]
            fdr.cutoffs = c()
            fdr.cutoffs[1] = max(pvals[pvals.a<0.01,i])
            fdr.cutoffs[2] = max(pvals[pvals.a<0.05,i])
            fdr.cutoffs[3] = max(pvals[pvals.a<0.10,i])
            fdr.cutoffs[4] = max(pvals[pvals.a<0.5,i])
            
            volcanopch = rep(1,dim(pvals)[1])
            volcanopch[pvals[,i]<fdr.cutoffs[4]]=16
            
            volcanocol=rep("azure4",dim(pvals)[1])
            volcanocol[pvals[,i]<fdr.cutoffs[4]]="chartreuse4"
            volcanocol[pvals[,i]<fdr.cutoffs[3]]="chartreuse3"
            volcanocol[pvals[,i]<fdr.cutoffs[2]]="chartreuse2"
            volcanocol[pvals[,i]<fdr.cutoffs[1]]="chartreuse1"
          }
          ### draw the volcano plot:
          plot(DEresults$ests[,i],-log10(pvals[,i]),xlab="log2(fold change)",ylab = "-log10(p-value)",main=newtermnames[covariates[i]],col=volcanocol,pch=volcanopch)   
          # draw lines for pval cutoffs if there's no FDR calculated:
          if(!is.element(pval.adjustment,c("BH","BY")))
          {
            abline(h=-log10(c(0.01,0.001)),lty=2)
            legend("bottomright",pch=c(16),col=c(codecols2[3]),legend = paste(gene.set.names[k],"genes"))
          }
          # draw lines for FDR cutoffs if FDR has been calculated:
          if(is.element(pval.adjustment,c("BH","BY")))
          {
            abline(h=-log10(fdr.cutoffs),lty=1:4)
            legend("bottomright",lty=c(NA,1:4),pch=c(16,rep(NA,4)),col=c(codecols2[3],rep(1,4)),
                   legend = c(paste(gene.set.names[k],"genes"),paste("FDR <",c("0.01","0.05","0.10","0.50"))))
          }
          points(DEresults$ests[gene.sets[,k]==1,i],-log10(pvals[gene.sets[,k]==1,i]),pch=16,col=codecols2[3])
          #legend("bottomright",pch=c(16,16),col=c(codecols2[3]),legend = paste(gene.set.names[k],"pathway genes"))
          sig.and.in.path = (pvals.all<0.01)&(gene.sets[,k]==1)
          if(sum(sig.and.in.path)>0){text(x=ests[sig.and.in.path,i],y=-log10(pvals[sig.and.in.path,i]),labels=dimnames(ests)[[1]][sig.and.in.path],cex=.8)}
          dev.off()}
        
        # write a csv of results for just genes in the pathway:
        select1 = (gene.sets[,k]==1)
        out = cbind(DEresults$ests[,i],DEresults$ests[,i]-1.96*DEresults$ses[,i],DEresults$ests[,i]+1.96*DEresults$ses[,i],
                    DEresults$pvals[,i],as.data.frame(DEresults$Pathways))
        extracolumn=FALSE
        if(pval.adjustment=="BY")
        {
          extracolumn=TRUE
          pvals.a = p.adjust(pvals[,i],method="BY")
          out = cbind(out,pvals.a); dimnames(out)[[2]][6] = "FDR"
        }
        if(pval.adjustment=="BH")
        {
          extracolumn=TRUE
          pvals.a = p.adjust(pvals[,i],method="BH")
          out = cbind(out,pvals.a); dimnames(out)[[2]][6] = "FDR"
        }
        if(pval.adjustment=="Bonferroni")
        {
          extracolumn=TRUE
          pvals.a = p.adjust(pvals[,i],method="bonferroni")
          out = cbind(out,pvals.a); dimnames(out)[[2]][6] = "Bonf. p-value"
        }
        out = out[select1,]
        out = out[order(out[,4]),]
        if(extracolumn){out = out[,c(1:4,6,5)]}
        if(!extracolumn){dimnames(out)[[2]] = c("Log2 fold change","Lower confidence limit","Upper confidence limit","P-value","Pathways")}
        if(extracolumn){dimnames(out)[[2]] = c("Log2 fold change","Lower confidence limit","Upper confidence limit","P-value",dimnames(out)[[2]][5],"Pathways")}
        out[,c(1:3)]=signif(out[,c(1:3)],3)
        out[,4]=signif(out[,4],4)
        if(extracolumn){out[,5]=signif(out[,5],4)} 
        
        write.csv(out,file=correct.filename(paste(path.to.GSA.results,"//Genes in ",dimnames(gene.sets)[[2]][k]," - ",make.names(dimnames(DEresults$ses)[[2]][i]),".csv",sep="")))
        
      } #end if(saveresults)
    }
  }
  
  
  if(saveresults)
  {
    
    print("Creating gene set analysis (GSA) results files")
    cat("LOG:Creating gene set analysis (GSA) results files",file=log,sep='\n\n',append=TRUE)
    cat("document.write('<p>Creating gene set analysis (GSA) results files</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
    
    # write global significance statistics:
    write.csv(round(global.sig.stats,3),file=paste(path.to.GSA.results,"//global significance stats - mean abs t-stat.csv",sep=""))
    write.csv(round(global.sig.stats.updown,3),file=paste(path.to.GSA.results,"//global significance stats - mean t-stat.csv",sep=""))
    bothstats = cbind(global.sig.stats,global.sig.stats.updown)
    colnames(bothstats) = paste(c(rep("Undirected",ncol(global.sig.stats)),rep("Directed",ncol(global.sig.stats.updown))),colnames(bothstats))
    write.csv(round(bothstats,3),file=paste(path.to.GSA.results,"//global significance stats - directed and undirected.csv",sep=""))
    
    if(dim(global.sig.stats)[2]>1)
    {
      
      ## heatmaps of path significance scores
      
	  # remove rows for any gene sets without results:
      global.sig.stats = global.sig.stats[!empty.gene.sets,]
      global.sig.stats.updown = global.sig.stats.updown[!empty.gene.sets,]

      # add line breaks to make column titles fit in the margins:
      global.sig.stats.linebreaks = global.sig.stats
      global.sig.stats.updown.linebreaks = global.sig.stats.updown
      dimnames(global.sig.stats.linebreaks)[[2]]=dimnames(global.sig.stats.updown.linebreaks)[[2]]=newtermnames.linebreak[covariates]
      
      for(r in 1:length(plottypearg)){
        
        plottype=plottypearg[r];
        tempfilename = drawplot(paste(path.to.GSA.results,"//heatmap of global significance scores",sep=""),plottype,width=1.3,height=1.3)
        tempfilename=gsub(path.results,"results",tempfilename)
        
        #par(mar=c(16,4,8,1))
        hmcols<-colorRampPalette(c("cornflowerblue","black","orange"))(256)
        heatmap.2(global.sig.stats.linebreaks,scale="none",symm=FALSE,trace="none",density.info="none",Colv=TRUE,margin=c(12,10),
                  cexCol = 0.2 + 1/log10(dim(global.sig.stats)[1]),col=hmcols)
        dev.off()
        tempfilename = drawplot(paste(path.to.GSA.results,"//heatmap of directed global significance scores - directed",sep=""),plottype,width=1.3,height=1.3)
        tempfilename=gsub(path.results,"results",tempfilename)
        
        #par(mar=c(16,4,8,1))
        hmcols<-colorRampPalette(c("blue","antiquewhite3",codecols2[4]))(256)
        breaks = seq(-max(abs(global.sig.stats.updown)),max(abs(global.sig.stats.updown)),length.out=length(hmcols)+1)
        heatmap.2(global.sig.stats.updown.linebreaks,scale="none",symm=FALSE,trace="none",density.info="none",Colv=TRUE,col=hmcols,
                  breaks=breaks,margin=c(12,10),cexCol = 0.2 + 1/log10(dim(global.sig.stats)[1]))
        dev.off()
      }
    } 
    print("Creating gene set analysis (GSA) HTML infrastructure")
    cat("LOG:Creating gene set analysis (GSA) HTML infrastructure",file=log,sep='\n\n',append=TRUE)
    cat("document.write('<p>Creating gene set analysis (GSA) HTML infrastructure</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
  } #end if(saveresults)
  
  
  out = list(mean.abs.tstats = global.sig.stats,mean.tstats = global.sig.stats.updown)
  print("Finished gene set analysis (GSA)")
  cat("LOG: Finished gene set analysis (GSA)",file=log,sep='\n\n',append=TRUE)
  cat("document.write('<p>Finished gene set analysis (GSA)</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
  return(out)
}
