
##############################################
### function to create a panel summarizing the arguments:
#
# 1) create the panel
# 2) add a heading for each section
# 3) add a paragraph with the section text
summarize.arguments.file = function()
{
  
  
  # Create the panel object that this function will return.
  argumentsPanel = panel(
    id = "arguments_panel", 
    title = "Summary of the options that were chosen when running this analysis.", 
    active = TRUE
    )

  
  # basic high-level choices:
  pg = ""
  pg = paste(pg,"Analysis name:",advanced.analysis.name,"\n")
  pg = paste(pg,"Location of results:",path.results.copy,"\n")
  pg = paste(pg,"Type of data:",data.type,"\n")
  pg = paste(pg,"File type of plots:",paste(plottypearg,collapse=","),"\n")
  
  addReportContent(
    element = argumentsPanel,
    heading(content = "Analysis Details"),
    paragraph(content = pg)
  )
  
  
  # Low Count Threshold Details
  pg = ""
  pg = paste(pg,"Remove Genes Below Specified Threshold:",prunegenes.run,"\n")
  pg = paste(pg,"Threshold count value:",prunegenes.bg.level,"\n")
  pg = paste(pg,"Remove genes below the threshold at frequency greater than:",prunegenes.missing.freq,"\n")
  
  addReportContent(
    element = argumentsPanel,
    heading(content = "Low Count Threshold Details"),
    paragraph(content = pg)
  )
  
  # sample annotation details
  pg = ""
  pg = paste(pg,"Unique sample identifier:",sample.annotation.id.column,"\n")
  for(i in 1:length(sampleannot.variables))
  {
    pg = paste(pg,"Covariate",i,": ",sampleannot.variables[i],"; Variable Type: ",sampleannot.variabletypes[i],"; Reference Level: ",sampleannot.referencelevels[i],"\n",sep="")
  }
  
  addReportContent(
    element = argumentsPanel,
    heading(content = "Sample Annotation Details"),
    paragraph(content = pg)
  )
  
  
  # gene annotation details
  pg = ""
  pg = paste(pg,"Column defining gene set membership:",gene.set.column,"\n")
  pg = paste(pg,"Gene annotation column giving cell type-specific genes:",celltypes.arg.column,"\n")
  
  addReportContent(
    element = argumentsPanel,
    heading(content = "Gene Annotation Details"),
    paragraph(content = pg)
  )
  
  
  # normalization details:
  pg = ""
  pg = paste(pg,"Perform normalization:",normmodule.arg.run,"\n")
  pg = paste(pg,"Number of housekeepers:",normmodule.arg.n.HKs,"\n")
  pg = paste(pg,"Auto-select number of housekeepers:",normmodule.arg.auto.HKs,"\n")

  addReportContent(
    element = argumentsPanel,
    heading(content = "Normalization Details"),
    paragraph(content = pg)
  )

  
  # DE details:
  pg = ""
  pg = paste(pg,"Perform differential expression testing:",DEmodule.arg.run,"\n")
  pg = paste(pg,"Predictors:",paste(DEmodule.arg.predictors,collapse=", "),"\n")
  pg = paste(pg,"Confounders:",paste(DEmodule.arg.confounders,collapse=", "),"\n")
  pg = paste(pg,"P-value adjustment:",DEmodule.arg.pval.adjustment,"\n")
  pg = paste(pg,"Run gene set analysis",gsamodule.arg.run,"\n")

  addReportContent(
    element = argumentsPanel,
    heading(content = "Differential Expression Analysis Details"),
    paragraph(content = pg)
  )
  
  
  # Pathview details:
  pg = ""
  pg = paste(pg,"Display results using Pathview:",pathviewmodule.arg.run,"\n")
  pathviewmodule.arg.view.colorscheme2 = pathviewmodule.arg.view.colorscheme
  if(pathviewmodule.arg.view.colorscheme=="DE"){pathviewmodule.arg.view.colorscheme2 = "Fold-change"}
  if(pathviewmodule.arg.view.colorscheme=="tstat"){pathviewmodule.arg.view.colorscheme2 = "t-statistic"}
  pg = paste(pg,"Color Pathview pathview plots by:",pathviewmodule.arg.view.colorscheme2,"\n")
  pg = paste(pg,"P-value threshold:",pathviewmodule.arg.pvalthres,"\n")
  pg = paste(pg,"Additional KEGG IDs:",paste(pathviewmodule.arg.keggIDs.custom,collapse=", "),"\n")
  
  addReportContent(
    element = argumentsPanel,
    heading(content = "Pathview  Details"),
    paragraph(content = pg)
  )
  
  
  # cell type scoring details:
  pg = ""
  pg = paste(pg,"Perform cell type profiling:",celltypes.arg.run,"\n")
  pg = paste(pg,"Covariates to include in plots:",paste(celltypes.arg.covariates,collapse=", "),"\n")
  pg = paste(pg,"Use all cell type genes or dynamically select a subset:",celltypes.arg.geneselection,"\n")
  pg = paste(pg,"P-value threshold for reporting a cell type's results:",celltypes.arg.pvalthreshold,"\n")
  pg = paste(pg,"Show results at the level of raw cell type abundance:",celltypes.arg.showraw,"\n")
  pg = paste(pg,"Show results at the level of relative cell type abundance:",celltypes.arg.showrelative,"\n")
  pg = paste(pg,"Use default contrasts for calculating relative cell type abundance:",celltypes.arg.usedefault,"\n")
  # <------------- need an argument for the path to the alternative cell tyep abundance
  
  addReportContent(
    element = argumentsPanel,
    heading(content = "Cell Type Profiling Details"),
    paragraph(content = pg)
  )
  
  
  # SGD details:
  pg = ""
  pg = paste(pg, "Run SGD analysis:", single.gene.analysis.run, "\n")
  for(i in 1:length(gene.names.to.describe))
  {
    pg = paste(pg, "Gene name ",i,": ", gene.names.to.describe[i], "\n")
  }  
  for(i in 1:length(describe.genes.by))
  {
    pg = paste(pg, "Grouping variable ",i,": ", describe.genes.by[i], "\n")
  }  
  pg = paste(pg, "Interval ID:", order.expression.trend.by, "\n")
  pg = paste(pg, "Series ID:", trend.expression.by, "\n") 
  pg = paste(pg, "Stratifying variable:", stratify.expression.trend.by, "\n")
  
  addReportContent(
    element = argumentsPanel,
    heading(content = "SGD Analysis Details"),
    paragraph(content = pg)
  )
  
  
  return(argumentsPanel)
}
