

###########################################################################
###########################################################################
# data exploration and QC
###########################################################################
###########################################################################


runQC = function(normalized,HKnormalized,HKs,covariates,covariates.type,gene.sets=NULL,annotcols,annotcols2,
                 codecols,path.to.QC.results,log,plottypearg,path.results,path.inc,path.to.csvs)  
{
  
  print("Starting QC")
  cat("LOG:Starting QC",file=log,sep='\n\n',append=TRUE)
  cat("document.write('<p>Starting QC</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
  
  alldata = cbind(normalized,HKnormalized)
  genecols = c(rep(codecols[1],dim(normalized)[2]),rep(codecols[2],dim(HKnormalized)[2]))
  genecols[is.element(dimnames(alldata)[[2]],HKs)]=codecols[3]
  
  # plot mean vs. sd in normalized data, color points by type: endog, HK (selected), HK (unselected)
  means = apply(alldata,2,mean)
  vars = apply(alldata,2,var)
  for(r in 1:length(plottypearg)){
    plottype=plottypearg[r];
    tempfilename = drawplot(filename=paste(path.to.QC.results,"//mean vs var for all genes",sep=""),plottype)
    tempfilename=gsub(path.results,"results",tempfilename)
    par(xpd=TRUE)
    par(mar=c(5,4,4,1))
    plot(vars~means,col=genecols,pch=(1+15*is.element(dimnames(alldata)[[2]],dimnames(HKnormalized)[[2]])),xlab="Mean(log2 expression)",ylab="Variance(log2 expression)")
    legend("topleft",inset=c(0,-0.16),col=codecols[c(1,3,2)],pch=c(1,16,16),legend=c("Endogenous genes","Housekeepers used in normalization","Housekeepers - unused"))
    topvars = order(vars,decreasing=TRUE)[1:20]
    text(means[topvars],vars[topvars],dimnames(alldata)[[2]][topvars])
    dev.off()}
  
  
  #### plot relationships btw covariates:
  print("Creating QC plots")
  cat("LOG:Creating QC plots",file=log,sep='\n\n',append=TRUE)
  cat("document.write('<p>Creating QC plots</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
  
  for(i in 1:(dim(covariates)[2]))
  {
    # make a univariate histogram or barplot:
    for(r in 1:length(plottypearg)){
      plottype=plottypearg[r];
      tempfilename = drawplot(filename=paste(path.to.QC.results,"//univariate plot of covariates - ",dimnames(covariates)[[2]][i],sep=""),plottype)
      tempfilename=gsub(path.results,"results",tempfilename)
      if(covariates.type[i]=="continuous")
      {
        hist(covariates[,i],xlab="",main=paste("Distribution of",colnames(covariates)[i]))
      }
      if(covariates.type[i]=="categorical")
      {
        barplot(table(covariates[,i]),main=paste("Distribution of",colnames(covariates)[i]),las=2,border=annotcols2[[i]])
      }  
      dev.off()
    }
  }
  if(dim(covariates)[2]>1)
  {
    # plot a matrix of results.  Scatterplots/boxplots/tables on top right, p-values for association on bottom left:
    for(i in 1:(dim(covariates)[2]-1))
    {
      for(j in (i+1):dim(covariates)[2])
      {
        for(r in 1:length(plottypearg)){
          plottype=plottypearg[r];
          #if(!(covariates.type[i]=="categorical")&(covariates.type[j]=="categorical"))
          #{
          tempfilename = drawplot(filename=paste(path.to.QC.results,"//pairs plot of covariates - ",dimnames(covariates)[[2]][i]," vs ",dimnames(covariates)[[2]][j],sep=""),plottype)
          tempfilename=gsub(path.results,"results",tempfilename)
          #}
          # scatterplot between continuous covariates:
          if((covariates.type[i]=="continuous")&(covariates.type[j]=="continuous"))
          {
            plot(covariates[,i],covariates[,j],xlab=dimnames(covariates)[[2]][i],ylab=dimnames(covariates)[[2]][j])
          }
          # boxplot between cont and cat:
          par(mar=c(5.1,6,4.1,2.1))
          if((covariates.type[i]=="continuous")&(covariates.type[j]=="categorical"))
          {
            par(mar=c(8,6,2,1))
            boxplot(covariates[,i]~covariates[,j],ylab=dimnames(covariates)[[2]][i],las=2,border=annotcols2[[j]],
                    main=paste(dimnames(covariates)[[2]][i],"vs.",dimnames(covariates)[[2]][j])) 
            
          }
          if((covariates.type[j]=="continuous")&(covariates.type[i]=="categorical"))
          {
            par(mar=c(8,6,2,1))
            boxplot(covariates[,j]~covariates[,i],ylab=dimnames(covariates)[[2]][j],las=2,border=annotcols2[[i]],
                    main=paste(dimnames(covariates)[[2]][j],"vs.",dimnames(covariates)[[2]][i]))  
          }
          # compare cat to cat:
          if((covariates.type[i]=="categorical")&(covariates.type[j]=="categorical"))
          {
            tab = as.matrix(table(covariates[,c(i,j)]))
            flip = dim(tab)[1]>dim(tab)[2]
            annotcodecol = i
            if(flip)
            {
              tab = t(tab)
              # color by i or j?
              annotcodecol = j
            }
            barplot(tab,col=annotcols2[[annotcodecol]],las=2,main=paste(dimnames(covariates)[[2]][j],"vs.",dimnames(covariates)[[2]][i]))  
            
            #write.csv(tab,file=paste(path.to.QC.results,"//pairs plot of covariates - ",dimnames(covariates)[[2]][i]," vs ",dimnames(covariates)[[2]][j],".csv",sep=""))
          }
          #if(!(covariates.type[i]=="categorical")&(covariates.type[j]=="categorical")){dev.off()}}          
          dev.off()}
      }
    }
  }
  write.csv(covariates,file="QC module - sample annot.csv")
  
  ### heatmap of all data, arranged by pathway membership  
  #hm1 = heatmap.2(t(normalized),scale="row")
  # first, fill in the row colors matrix:
  if(length(covariates)>0)
  {
    rowcols = annotcols
    cat("dimnames(covariates):",file=log,sep='\n',append=TRUE)
    suppressWarnings(write.table(dimnames(covariates)[[2]],file=log,sep='\t',append=TRUE))
    suppressWarnings(write.table(dimnames(covariates)[[1]][1:5],file=log,sep='\t',append=TRUE))
    cat("dim(rowcols)",file=log,sep='\n',append=TRUE)
    cat(dim(rowcols),file=log,sep='\n',append=TRUE)
    dimnames(rowcols)=dimnames(covariates)
    #if(i==1){rowcols = cbind(rowcols,"white")}
    #if(i==1){rowcols = cbind(rowcols,rowcols)}
  }
  #  if(length(gene.sets)>0)
  #  {
  #    bigmatrix = t(normalized)
  #    pathsizes = dim(bigmatrix)[1]
  #  }
  if(length(gene.sets)>0)
  {
    #    bigmatrix = c()
    #    pathsizes = c()
    #    pathbreaks = c()
    #    pathcolors = c()
    gene.sets = gene.sets[match(dimnames(normalized)[[2]],dimnames(gene.sets)[[1]]),]
    for(k in 1:dim(gene.sets)[2])
    {
      print(paste("Creating QC heatmaps for gene set ",k,"/",dim(gene.sets)[2]))
      cat(paste("LOG: Creating QC heatmaps for gene set ",k,"/",dim(gene.sets)[2]),file=log,sep='\n\n',append=TRUE)
      cat(paste("document.write('<p>Creating QC heatmaps for gene set ",k,"/",dim(gene.sets)[2],"</p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
      
      # isolate data matrix from the current gene set:
      pathmatrix = t(normalized[,dimnames(gene.sets)[[1]][gene.sets[,k]==1]])
      # run a heatmap on the pathway matrix to get a better ordering of the genes
      for(r in 1:length(plottypearg)){
        plottype=plottypearg[r];
        tempfilename = drawplot(filename=paste(path.to.QC.results,"//heatmap of data - ",make.names(dimnames(gene.sets)[[2]][k]),sep=""),plottype,
                                width=(1.5+1*(dim(normalized)[1]>50)),height=1.75,heatmapres=TRUE)
        #width=(3+2*(dim(normalized)[1]>50)),height=3.5)
        tempfilename=gsub(path.results,"results",tempfilename)
        par(mar=c(10,4,2,1))
        hmcols<-colorRampPalette(c("cornflowerblue","black","orange"))(256)
        if(length(covariates)>0){ heatmap.3(pathmatrix,symm=FALSE,Colv=TRUE,Rowv = TRUE,ColSideColors=rowcols,margins = c(8,8),col=hmcols,scale="row") }
        if(length(covariates)==0){ heatmap.3(pathmatrix,symm=FALSE,Colv=TRUE,Rowv = TRUE,col=hmcols,scale="row") }
        dev.off()}
      
      #### correlation matrix of all samples (just the pathway's data):  
      cormat = cor((pathmatrix))
      #neg.elements = sign(cormat)==-1
      #cormat.rescaled = cormat/abs(min(cormat))
      #cormat2 = replace(cormat,neg.elements,cormat.rescaled[neg.elements])
      for(r in 1:length(plottypearg)){
        plottype=plottypearg[r];
        tempfilename = drawplot(filename=paste(path.to.QC.results,"//correlation between samples - heatmap - ",make.names(dimnames(gene.sets)[[2]][k]),sep=""),plottype,width=2,height=2,heatmapres=TRUE)
        tempfilename=gsub(path.results,"results",tempfilename)
        par(mar=c(10,4,2,1))
        hmcols<-colorRampPalette(c("blue","antiquewhite3",codecols2[4]))(256)
        hmcols = hmcols
        breaks = seq(-1,max(abs(cormat)),length.out=length(hmcols)+1)
        #if(length(covariates)>0){ heatmap.3(cormat,symm=TRUE,margins=c(8,8),ColSideColors=(rowcols),RowSideColors=(rowcols)) }
        if(length(covariates)>0){ heatmap.3(cormat,symm=TRUE,margins=c(8,8),RowSideColors=t(rowcols),ColSideColors=(rowcols),col=hmcols,Colv=TRUE) }
        if(length(covariates)==0){ heatmap.3(cormat,symm=TRUE,margins=c(8,8),col=hmcols,Colv=TRUE) }
        dev.off()}
      
      # and draw a PCA plot (just the pathway data):
      pc = prcomp(t(pathmatrix))
      for(i in 1:dim(covariates)[2])
      {
        pointcols = annotcols[,i]
        for(r in 1:length(plottypearg)){
          plottype=plottypearg[r];
          tempfilename = drawplot(filename=paste(path.to.QC.results,"//principal components colored by",dimnames(covariates)[[2]][i]," - ",make.names(dimnames(gene.sets)[[2]][k]),sep=""),plottype)
          tempfilename=gsub(path.results,"results",tempfilename)
          
          pch = 1+15*(nrow(pc$x)<100)
          pairs(pc$x[,1:min(4,dim(pc$x)[2])],col=pointcols,xaxt="n",yaxt="n",pch=pch,
                labels = paste("PC",1:4," - ",round((pc$sdev^2)[1:4]/sum(pc$sdev^2),2),sep=""))
          dev.off()}
        #for(r in 1:length(plottypearg)){
        #  plottype=plottypearg[r];
        #  tempfilename = drawplot(filename=paste(path.to.QC.results,"//principal components colored by",dimnames(covariates)[[2]][i]," legend",sep=""),plottype,width=.5)
        #  tempfilename=gsub(path.results,"results",tempfilename)
        #  frame()
        #  legend("center",pch=1,col=c(NA,"deepskyblue2","firebrick2"),legend=c(main=names(covariates)[i],"low","high"))
        #  dev.off()}      
      }
      
      
    }
  }
  
  # plot the final heatmap:
  #dimnames(pathcolors)[[2]]=c("","")
  for(r in 1:length(plottypearg)){
    plottype=plottypearg[r];
    tempfilename = drawplot(filename=paste(path.to.QC.results,"//heatmap of all genes X samples",sep=""),plottype,height=2,width=1.5,heatmapres=TRUE)
    #tempfilename = drawplot(filename=paste(path.to.QC.results,"//heatmap of all genes X samples",sep=""),plottype,height=4,width=3)
    tempfilename=gsub(path.results,"results",tempfilename)
    par(mar=c(10,8,2,1))
    hmcols<-colorRampPalette(c("cornflowerblue","black","orange"),bias=1.0)(256)
    if(length(covariates)>0){ heatmap.3(t(normalized),symm=FALSE,Colv=TRUE,Rowv = TRUE,labRow=NA,margins = c(8,8),ColSideColors=(rowcols),col=hmcols,scale="row") }
    if(length(covariates)==0){ heatmap.3(t(normalized),symm=FALSE,Colv=TRUE,Rowv = TRUE,labRow=NA,margins = c(8,8),col=hmcols,scale="row") }
    dev.off()}
  
  
  #### correlation matrix of all samples:  
  cormat = cor(t(normalized))
  #neg.elements = sign(cormat)==-1
  #cormat.rescaled = cormat/abs(min(cormat))
  #cormat2 = replace(cormat,neg.elements,cormat.rescaled[neg.elements])
  #write.csv(cormat,file = paste(path.to.csvs,"//QC - correlation matrix of samples.csv",sep=""))
  for(r in 1:length(plottypearg)){
    plottype=plottypearg[r];
    tempfilename = drawplot(filename=paste(path.to.QC.results,"//correlation between samples - heatmap",sep=""),plottype,width=2,height=2,heatmapres=TRUE)
    #tempfilename = drawplot(filename=paste(path.to.QC.results,"//correlation between samples - heatmap",sep=""),plottype,width=4,height=4)
    tempfilename=gsub(path.results,"results",tempfilename)
    par(mar=c(10,4,2,1))
    hmcols<-colorRampPalette(c("blue","antiquewhite3",codecols2[4]))(256)
    breaks = seq(-1,1,length.out=length(hmcols)+1)
    #if(length(covariates)>0){ heatmap.3(cormat,symm=TRUE,margins=c(8,8),ColSideColors=(rowcols),RowSideColors=(rowcols)) }
    if(length(covariates)>0){ heatmap.3(cormat,symm=TRUE,margins=c(8,8),RowSideColors=t(rowcols),ColSideColors=(rowcols),col=hmcols,Colv=TRUE) }
    if(length(covariates)==0){ heatmap.3(cormat,symm=TRUE,margins=c(8,8),col=hmcols,Colv=TRUE) }
    dev.off()}
  
  
  ### PCA-based QC:
  pc = prcomp(normalized)
  ## automatically identify outliers in PCA:
  x = pc$x[,1:min(4,dim(pc$x)[2])]
  
  # outliers: anything > 3sds from mean:
  outliers = matrix(FALSE,dim(x)[1],dim(x)[2])
  dimnames(outliers)[[1]]=dimnames(normalized)[[1]]
  dimnames(outliers)[[2]] = paste("Outlier on PC",1:dim(x)[[2]])
  for(i in 1:dim(x)[2])
  {
    outliers[abs(x[,i]-mean(x[,i]))>3*sd(x[,i]),i]=TRUE   
  }
  # output list of outliers along with PC on which they're an outlier:
  if(sum(rowSums(outliers)>0)>1){outliers = outliers[rowSums(outliers)>0,]}
  if(sum(rowSums(outliers)>0)==1)
  {
    outliername = dimnames(outliers)[[1]][rowSums(outliers)>0] 
    outliers = t(as.matrix(outliers[rowSums(outliers)>0,]))
    dimnames(outliers)[[1]]=outliername
  }
  if(sum(rowSums(outliers)>0)>0){write.csv(outliers,file=paste(path.to.QC.results,"//outliers in first 4 principal components.csv",sep=""))}
  
  ## PC pairs plots colored by covariates:   #<---- modification: now using annotcols to harmonize colors.  if(FALSE) after this code chunk was the old coloring scheme
  #covariates = as.data.frame(covariates)
  if(dim(covariates)[2]>0)
  {
    for(i in 1:dim(covariates)[2])
    {
      pointcols = annotcols[,i]
      for(r in 1:length(plottypearg)){
        plottype=plottypearg[r];
        tempfilename = drawplot(filename=paste(path.to.QC.results,"//principal components colored by",dimnames(covariates)[[2]][i],sep=""),plottype)
        tempfilename=gsub(path.results,"results",tempfilename)
        pch = 1+15*(nrow(pc$x)<100)
        pairs(pc$x[,1:min(4,dim(pc$x)[2])],col=pointcols,xaxt="n",yaxt="n",pch=pch,
              labels = paste("PC",1:4," - ",round((pc$sdev^2)[1:4]/sum(pc$sdev^2),2),sep=""))
        dev.off()}
      #for(r in 1:length(plottypearg)){
      #  plottype=plottypearg[r];
      #  tempfilename = drawplot(filename=paste(path.to.QC.results,"//principal components colored by",dimnames(covariates)[[2]][i]," legend",sep=""),plottype,width=.5)
      #  tempfilename=gsub(path.results,"results",tempfilename)
      #  frame()
      #  legend("center",pch=1,col=c(NA,"deepskyblue2","firebrick2"),legend=c(main=names(covariates)[i],"low","high"))
      #  dev.off()}      
    }
    write.csv(covariates,file=paste(path.to.csvs,"//QC - covariates used in p-value histograms.csv",sep=""))
    
    ### p-value histograms of univariate analyses of ALL variables:
    for(r in 1:length(plottypearg)){
      print(paste("Creating QC histograms ",r,"/",length(plottypearg)))
      cat(paste("LOG: Creating QC histograms ",r,"/",length(plottypearg)),file=log,sep='\n\n',append=TRUE)
      cat(paste("document.write('<p>Creating QC histograms ",r,"/",length(plottypearg),"</p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
      plottype=plottypearg[r];
      
      nrows = ceiling(ncol(covariates)/2)
      ncols = ceiling(ncol(covariates)/2)
      
      tempfilename = drawplot(filename=paste(path.to.QC.results,"//p-value histograms",sep=""),plottype,height=nrows*.8,width=ncols*.8)
      tempfilename=gsub(path.results,"results",tempfilename)    
      #if(dim(covariates)[[2]]<=4)
      #{
      #  par(mfrow=c(ceiling(dim(covariates)[[2]]/2),min(dim(covariates)[[2]],2)))
      #}
      #if(dim(covariates)[[2]]>4)
      #{
      #  par(mfrow=c(ceiling(dim(covariates)[[2]]/3),min(dim(covariates)[[2]],3)))
      #}
      par(mfrow=c(nrows,ncols))
      for(i in 1:dim(covariates)[2])
      {
        pvals = c()
        for(j in 1:dim(normalized)[2])
        {
          mod = lm(cbind(normalized[,j],as.data.frame(covariates[,i])))      
          mod2 = lm(normalized[,j]~1) 
          # run LRT:
          LLR = logLik(mod)-logLik(mod2)
          df = length(mod$coef)-length(mod2$coef)
          pvals[j]=1-pchisq(2*LLR,df)
        }
        hist(pvals,main=paste("p-value histogram:\n",dimnames(covariates)[[2]][i],sep=""),breaks=(0:20)/20,xlim=c(0,1),cex.main=1.45,cex.lab=1.45,xlab="p-values")
      }
      dev.off()}
  }
  print("Creating QC HTML infrastructure")
  cat("LOG: Creating QC HTML infrastructure",file=log,sep='\n\n',append=TRUE)
  cat("document.write('<p>Creating QC HTML infrastructure</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
  
  print("Finished QC")
  cat("LOG:Finished QC",file=log,sep='\n\n',append=TRUE)
  cat("document.write('<p>Finished QC</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
} 
