

###########################################################################
###########################################################################
#               Normalization
###########################################################################
###########################################################################

#### function to normalize the raw data given a list of HKs:
normalize.given.HKs = function(raw,HKdata,HKs)
{   
  # use only the selected HKs:
  HKdata.selected = HKdata[,is.element(dimnames(HKdata)[[2]],HKs)]
  # get mean of HKs
  HKmeans = apply(HKdata.selected,1,mean)
  # apply norm factors:
  norm.factors = HKmeans-mean(HKmeans)
  normalized = raw-norm.factors
  HKnormalized = HKdata-norm.factors
  out = list(normalized=normalized,HKnormalized=HKnormalized,norm.factors=norm.factors)
  return(out)
}

normalizerawdata = function(raw,method=c("choose HKs","none"),choose.method = c("geNorm"),
                            n.HKs = 10,auto.HKs,codeset.HKs,
                            path.to.normalization.results,log,plottypearg,path.results,path.inc)
{
  print("Starting normalization")
  cat("LOG:Starting normalization",file=log,sep='\n\n',append=TRUE)
  #cat("document.write('<p>Starting normalization</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
  
  codeset.genes = dimnames(raw)[[2]]
  
  ### basic processing: threshold and log-transform:
  raw = replace(raw,raw<1,1)
  raw = log2(raw)
  ### split data into HKs and informative genes:
  # (1st condition the codeset.HKs variable)
  #codeset.HKs[is.na(codeset.HKs)]=0
  #cat("table(codeset.HKs):",file=log,sep='\n',append=TRUE)
  #cat(table(codeset.HKs),file=log,sep='\n',append=TRUE)
  #cat("sum(codeset.HKs):",file=log,sep='\n',append=TRUE)
  #cat(sum(codeset.HKs),file=log,sep='\n',append=TRUE)
  suppressWarnings(write.table(table(codeset.HKs==1),file=log,sep='\t',append=TRUE))
  suppressWarnings(write.table(table(codeset.HKs),file=log,sep='\t',append=TRUE))
  cat("dim(raw):",file=log,sep='\t',append=TRUE)
  cat(dim(raw),file=log,sep='\t',append=TRUE)
  HKdata = raw[,codeset.HKs]
  raw = raw[,setdiff(dimnames(raw)[[2]],codeset.HKs)]
  ### if it's already normalized data, do nothing:
  if(method=="none")
  {
    warning("Warning: No normalization will be performed - assuming the input data is already normalized")
    cat("LOG: Warning: No normalization will be performed - assuming the input data is already normalized",file=log,sep='\n\n',append=TRUE)
    cat("document.write('<p>Warning: No normalization will be performed - assuming the input data is already normalized</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
    normalized = raw
    HKnormalized = HKdata
    HKs = NULL
    normalizedall = list(); normalizedall[[1]] = rep(1,dim(raw)[1])
  }
  ### if a list of HKs to use is input, check data:
  #  if(method == "specific HKs")
  #  {
  #    # make sure a vector of HKs has been input.  If not, go ahead with the geNorm algorithm:
  #    if(length(selected.HKs)==0)
  #    {
  #      warning("Warning: No housekeepers were preselected - using the geNorm algorithm to select housekeepers")
  #      cat("LOG: Warning: No housekeepers were preselected - using the geNorm algorithm to select housekeepers",file=log,sep='\n\n',append=TRUE)
  #      #cat("document.write('<p>Warning: No housekeepers were preselected - using the geNorm algorithm to select housekeepers</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
  #      method="choose HKs"; choose.method = "geNorm"
  #    }
  #    # make the vector of HKs belongs to the codeset:
  #    chosen.outside.codeset = unique(setdiff(selected.HKs,codeset.genes))
  #    if(length(chosen.outside.codeset)>0)
  #    {
  #      warning(paste("Warning: The following genes do not appear in the codeset:",paste(chosen.outside.codeset,collapse=""),". Using the remaining ",length(unique(selected.HKs))-length(unique(chosen.outside.codeset))," preselected housekeepers.",sep=""))
  #      cat(paste("LOG: Warning: The following genes do not appear in the codeset:",paste(chosen.outside.codeset,collapse=""),". Using the remaining ",length(unique(selected.HKs))-length(unique(chosen.outside.codeset))," preselected housekeepers.",sep=""),file=log,sep='\n\n',append=TRUE)
  #      cat(paste("document.write('<p>Warning: The following genes do not appear in the codeset:",paste(chosen.outside.codeset,collapse=""),". Using the remaining ",length(unique(selected.HKs))-length(unique(chosen.outside.codeset))," preselected housekeepers.</p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
  #      selected.HKs=intersect(selected.HKs,codeset.genes)
  #    }
  #    chosen.outside.HKcandidates = unique(setdiff(selected.HKs,codeset.HKs))
  #    if(length(chosen.outside.HKcandidates)>0)
  #    {
  #      warning(paste("Warning: The list of selected housekeepers includes the following genes chosen as informative in cancer: ",
  #                    paste(chosen.outside.HKcandidates,collapse="")))
  #      cat(paste("LOG: Warning: The list of selected housekeepers includes the following genes chosen as informative in cancer: ",
  #                paste(chosen.outside.HKcandidates,collapse="")),file=log,sep='\n\n',append=TRUE)
  #      cat(paste("document.write('<p>Warning: The list of selected housekeepers includes the following genes chosen as informative in cancer: ",
  #                paste(chosen.outside.HKcandidates,collapse=""),"</p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
  #    }
  #  }
  #  ### if a list of HKs to use is input, use those HKs to normalize the data:
  #  if(method == "specific HKs")
  #  {
  #    ## use only the selected HKs:
  #    normalizedall = normalize.given.HKs(raw,HKdata,selected.HKs) 
  #    normalized = normalizedall$normalized
  #    HKnormalized = normalizedall$HKnormalized
  #    HKs = selected.HKs
  #  }
  ### select normalization genes in a data-driven manner:
  if(method == "choose HKs")
  {
    # make sure there's an allowed method chosen:
    if(!is.element(choose.method,c("geNorm")))
    {
      warning("Warning: No method for choosing housekeepers selected - using the geNorm algorithm")
      cat("LOG: Warning: No method for choosing housekeepers selected - using the geNorm algorithm",file=log,sep='\n\n',append=TRUE)
      cat("document.write('<p>Warning: No method for choosing housekeepers selected - using the geNorm algorithm</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
      choose.method="geNorm"
    }
    
    if(choose.method=="geNorm")
    {
      minNrHKs = 2
      # run geNorm:
      temp = selectHKs(HKdata,method="geNorm",minNrHKs=minNrHKs,log=TRUE,Symbols = dimnames(HKdata)[[2]],trace=FALSE)
      ## select top number of genes:
      # if you decided beforehand how many to take:
      if(!auto.HKs)
      {
        HKs = as.vector(temp$ranking)[1:n.HKs]
      }
      # if you want to choose the top housekeepers dynamically:
      nselect=n.HKs
      if(auto.HKs)
      {
        nselect = dim(HKdata)[2]-(1:length(temp$variation))[temp$variation==min(temp$variation)]+1
        # require at least 6 HKs:
        nselect = max(nselect,6)
        HKs = as.vector(temp$ranking)[1:nselect] 
      }
      
      
      # plot the results as a sanity check:
      for(r in 1:length(plottypearg)){
        plottype=plottypearg[r];
        tempfilename = drawplot(filename=paste(path.to.normalization.results,"//HK selection details - pairwise variance",sep=""),plottype)
        tempfilename=gsub(path.results,"results",tempfilename)
        
        plot(temp$variation,
             col=c(rep("grey",dim(HKdata)[2]-nselect),rep("black",nselect)),
             pch=16,
             main="Genes selected using geNorm",
             ylab="Pairwise variation during stepwise selection",xlab="Order removed")
        abline(v=dim(HKdata)[2]-nselect+.5)
        legend("topleft",col=c("black","grey"),pch=c(16,1),legend=c("selected","unselected"))
        dev.off()}
      
    }
    
    # now normalize:
    normalizedall = normalize.given.HKs(raw,HKdata,HKs) 
    normalized = normalizedall$normalized
    HKnormalized = normalizedall$HKnormalized
    
    # plot sample means before and after normalization to show how successful it was:
    for(r in 1:length(plottypearg)){
      plottype=plottypearg[r];
      tempfilename = drawplot(filename=paste(path.to.normalization.results,"//total expression before and after normalization",sep=""),plottype,height=1.5)
      tempfilename=gsub(path.results,"results",tempfilename)
      par(mfrow=c(2,1))
      hist(apply(raw,1,mean),xlim=range(apply(raw,1,mean)),main="Raw data",xlab="Samples' mean log expression of all genes",breaks=20)
      hist(apply(cbind(normalized,HKnormalized),1,mean),xlim=range(apply(raw,1,mean)),main="Normalized data",xlab="Samples' mean log expression of all genes",breaks=20)
      dev.off()}
  }
  print("Normalized data")
  cat("LOG:Normalized data",file=log,sep='\n\n',append=TRUE)
  cat("document.write('<p>Normalized data</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
  cat("document.write('  	  	<ul>');\n",file=paste(path.inc,"//panel2_5.js",sep=""),append=TRUE)
  
  
  # write csv of selected HKs:
  if(method!="none")
  {
    HKstowrite = as.data.frame(HKs)
    names(HKstowrite) = "HKs in the order by which they were selected"
    write.csv(HKstowrite,file=paste(path.to.normalization.results,"//selected housekeepers.csv",sep=""),row.names=FALSE)
    
    file=paste("results//Normalization","//selected housekeepers.csv",sep="")
    strTemp=paste("document.write('	       <li><a href=\"",file,"\">View Selected HK Genes</a></li>')\n",sep="")
    cat(strTemp,file=paste(path.inc,"//panel2_5.js",sep=""),append=TRUE)
    
  }
  # threshold normalized data at 1:
  normalized = replace(normalized,normalized<0,0)
  HKnormalized = replace(HKnormalized,HKnormalized<0,0)
  out = list(normalized=normalized,HKnormalized=HKnormalized,HKs=HKs,norm.factors=normalizedall$norm.factors)
  # write csv of normalized data:
  write.csv(normalized,file=paste(path.to.normalization.results,"//normalized log2 data.csv",sep=""))
  
  print("Created normalized data files")
  cat("LOG:Created normalized data files",file=log,sep='\n\n',append=TRUE)
  cat("document.write('<p>Created normalized data files</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
  return(out)
}
