
#################################
#################################
## pathview
#################################
#################################

# color KEGG maps by differential expression: 
# options:  by fold-change, by signed log p-value, by test stat
run.pathview = function(DEresult, keggIDs=c("04330","04310","04340","04350","04010","04630","04151","04014","04210","04110"),
                        colorscheme = c("tstat","estimate"),pvalthresh=0.05, view=c("KEGG","graphviz"),
                        kegg.dir=NULL,xml.files=NULL,predictors=NULL,path.to.pathview.results,
                        log,path.results,path.inc,path.to.csvs)   
{
  
  
  
  J = DEresult$npredictors
  print("Starting pathview analysis")
  cat("LOG:Starting pathview analysis",file=log,sep='\n\n',append=TRUE)
  cat("document.write('<p>Starting pathview analysis</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
  
  
  if(colorscheme=="tstat")
  {
    wts = as.matrix(DEresult$ests[,1:J]/DEresult$ses[,1:J])
  }
  if(colorscheme=="DE")  
  {
    wts = as.matrix(DEresult$ests[,1:J]) 
  }
  # truncate the weights:
  wts = wts*(DEresult$pvals[,1:J]<pvalthresh)
  dimnames(wts)[[2]] = dimnames(DEresult$ests)[[2]][1:J]
  
  colnames(wts)=make.names(colnames(wts))
  
  write.csv(wts,file=paste(path.to.csvs,"//pathview - weights.csv",sep=""))
  write.csv(DEresult$pvals,file=paste(path.to.csvs,"//pathview - pvals.csv",sep=""))
  write.csv(DEresult$ests,file=paste(path.to.csvs,"//pathview - ests.csv",sep=""))
  
  d=FALSE
  
  maindir = getwd()
  setwd(path.to.pathview.results)
  for(j in 1:J)
  {
    # loop through specified keggIDs:
    if(length(keggIDs)>0)
    {
      for(k in 1:length(keggIDs))
      {
        if(!d){
          print(paste("Downloading KEGG files for", keggIDs[k]," pathway ",k,"/",length(keggIDs)))
          cat(paste("LOG:Downloading KEGG files for", keggIDs[k]," pathway ",k,"/",length(keggIDs)),file=log,sep='\n\n',append=TRUE)
          cat(paste("document.write('<p>Downloading KEGG files for", keggIDs[k]," pathway ",k,"/",length(keggIDs)," </p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
        }
        if(colorscheme=="DE"){genelimit = c(-2.5,2.5)}
        if(colorscheme=="tstat"){genelimit = c(-10,10)}
        
        # run pathview for the appropriate ID:
        if(view=="KEGG")
        {
          pv <- suppressMessages(suppressWarnings(try(pathview(gene.data = wts[,j], cpd.data=NULL,pathway.id = keggIDs[k], species = pathview.species, out.suffix = make.names(dimnames(wts)[[2]][j]),
                                                               kegg.native = TRUE,limit=list(gene=genelimit,cpd=c(-1,1)),gene.idtype="Symbol",multi.state=FALSE,same.layer=TRUE,
                                                               discrete =list(gene=FALSE, cpd=FALSE)))))
        }
        if(view=="graphviz")
        {
          pv <- suppressMessages(suppressWarnings(try(pathview(gene.data = wts[,j], cpd.data=NULL,pathway.id = keggIDs[k], species = pathview.species, out.suffix = make.names(dimnames(wts)[[2]][j]),
                                                               kegg.native = FALSE,limit=list(gene=genelimit,cpd=c(-1,1)),gene.idtype="Symbol",multi.state=FALSE,same.layer=FALSE))))  
        }
      }
      d=TRUE
      print(paste("Creating pathview analysis results ",j,"/",J))
      cat(paste("LOG: Creating pathview analysis results ",j,"/",J),file=log,sep='\n\n',append=TRUE)
      cat(paste("document.write('<p>Creating pathview analysis results ",j,"/",J," </p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
    }
  }
  setwd(maindir)
  print("Finished pathview analysis")
  cat("LOG:Finished pathview analysis",file=log,sep='\n\n',append=TRUE)
  cat("document.write('<p>Finished pathview analysis</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
}